#!/usr/bin/env python
# -*- coding: utf-8 -*-
from maser.tools.cdf.cdfcompare.subparser import add_cdfcompare_subparser  # noqa: F401
from maser.tools.cdf.cdfcompare.cdf_compare import cdf_compare  # noqa: F401
